/*
	defines.h
	---------

	This file houses a ton of macro definitions, mostly for GameBoy tile ID's,
	however it also contains some game-specific macro definitions as well.

	Note that, many of these macros are never actually used in game, but since
	they're defined as macros they won't consume rom size if they aren't used.
	Thus, this file contains a thorough list of tile definitions even if unused.

	Finally, this final also defines some global variables and arrays to use
	along side the macros to make tile swapping easier later on.
*/


/*
	Main game color defines

	This will use the same numbers as their corresponding palette
*/
#define EMPTY 0
#define GREEN 1
#define BLUE 2
#define YELLOW 3
#define PURPLE 4
#define RED 5


/*
	Sprite Palette Defines:

	These are the main colour pallete for the entire game.

	Since dot's is pretty simple, hopefully we can squeek by with just these.

	The PAL_COL entries change tiles for the games main dot colors,
	i.e. green, blue, yellow, purple, and red.

	Both the dots, lines, and arrow sprites will use these same palettes.
*/
#define PAL_GRAYSCALE 0
#define PAL_COL_GREEN 1
#define PAL_COL_BLUE 2
#define PAL_COL_YELLOW 3
#define PAL_COL_PURPLE 4
#define PAL_COL_RED 5
#define PAL_TITLE_1 6
#define PAL_TITLE_2 7


// palettes for games BG tiles
UWORD bgTilePalettes[] = {
	MainTilesCGBPal0c0, MainTilesCGBPal0c1, MainTilesCGBPal0c2, MainTilesCGBPal0c3,
	MainTilesCGBPal1c0, MainTilesCGBPal1c1, MainTilesCGBPal1c2, MainTilesCGBPal1c3,
	MainTilesCGBPal2c0, MainTilesCGBPal2c1, MainTilesCGBPal2c2, MainTilesCGBPal2c3,
	MainTilesCGBPal3c0, MainTilesCGBPal3c1, MainTilesCGBPal3c2, MainTilesCGBPal3c3,
	MainTilesCGBPal4c0, MainTilesCGBPal4c1, MainTilesCGBPal4c2, MainTilesCGBPal4c3,
	MainTilesCGBPal5c0, MainTilesCGBPal5c1, MainTilesCGBPal5c2, MainTilesCGBPal5c3,
	MainTilesCGBPal6c0, MainTilesCGBPal6c1, MainTilesCGBPal6c2, MainTilesCGBPal6c3,
	MainTilesCGBPal7c0, MainTilesCGBPal7c1, MainTilesCGBPal7c2, MainTilesCGBPal7c3
};


/*
	These are the background color integers for the first color of pallets,
	for when we need to completely change the games background color
	during square-completion mode
*/
#define bgColorWhite 32767
#define bgColorGreen 31739
#define bgColorBlue 32732
#define bgColorYellow 29695
#define bgColorPurple 32670
#define bgColorRed 29599


// array of BG colors so we can reference them by index
int BG_COLORS[] =
{
	bgColorWhite,
	bgColorGreen,
	bgColorBlue,
	bgColorYellow,
	bgColorPurple,
	bgColorRed
};


/*
	Sprite Tile Defines:

	These are the indicies for the sprite tiles, not sprites themselves
*/
#define ST_ARROW_HORIZ 41
#define ST_ARROW_VERT 42
#define ST_HAND_TL 43
#define ST_HAND_BL 44
#define ST_HAND_TR 45
#define ST_HAND_BR 46


/*
	Title Screen Tile Defines:

	These are the custom tiles just to display the title screen:
*/
#define TL_TITLE_LETTERS_0 0
#define TL_TITLE_LETTERS_1 1
#define TL_TITLE_LETTERS_2 2
#define TL_TITLE_LETTERS_3 3
#define TL_TITLE_LETTERS_4 4
#define TL_TITLE_LETTERS_5 5
#define TL_TITLE_LETTERS_6 6
#define TL_TITLE_LETTERS_7 7
#define TL_TITLE_LETTERS_8 8
#define TL_TITLE_LETTERS_9 9
#define TL_TITLE_LETTERS_10 10
#define TL_TITLE_LETTERS_11 11
#define TL_TITLE_LETTERS_12 12
#define TL_TITLE_LETTERS_13 13
#define TL_TITLE_LETTERS_14 14
#define TL_TITLE_LETTERS_15 15
#define TL_TITLE_LETTERS_16 16
#define TL_TITLE_DOTS_1 17
#define TL_TITLE_DOTS_2 18
#define TL_TITLE_DOTS_3 19
#define TL_TITLE_DOTS_4 20


/*
	Text Tiles - no, not cloths.
*/
#define TL_TEXT_P 21
#define TL_TEXT_R 22
#define TL_TEXT_E 23
#define TL_TEXT_S 24
#define TL_TEXT_T 25
#define TL_TEXT_A 26
#define TL_TEXT_I 27
#define TL_TEXT_M 28
#define TL_TEXT_C 29
#define TL_TEXT_COLON 30
#define TL_TEXT_0 31
#define TL_TEXT_1 32
#define TL_TEXT_2 33
#define TL_TEXT_3 34
#define TL_TEXT_4 35
#define TL_TEXT_5 36
#define TL_TEXT_6 37
#define TL_TEXT_7 38
#define TL_TEXT_8 39
#define TL_TEXT_9 40
#define TL_TEXT_B 54
#define TL_TEXT_U 55
#define TL_TEXT_L 56
#define TL_TEXT_D 59


/*
	char version of the TL_EMPTY because some code will need to pass it in as
	a pointer (i.e. `&TL_NONE`), which would not work with macros
*/
unsigned char TL_NONE = 0;


// array of digits so we can programatically set these tiles via array index
unsigned char TL_DIGITS[] =
{
	TL_TEXT_0,	// 0
	TL_TEXT_1,	// 1
	TL_TEXT_2,	// 2
	TL_TEXT_3,	// 3
	TL_TEXT_4,	// 4
	TL_TEXT_5,	// 5
	TL_TEXT_6,	// 6
	TL_TEXT_7,	// 7
	TL_TEXT_8,	// 8
	TL_TEXT_9	// 9
};


/*
	Game Field Tiles
*/
#define TL_EMPTY 0
#define TL_LINE_VERT 47
#define TL_LINE_HORIZ 48
#define TL_DOT_BIG_CENTER 49
#define TL_DOT_BIG_TOP_AND_BOTTOM 50
#define TL_DOT_BIG_LEFT_AND_RIGHT 51
#define TL_DOT_BIG_CORNER 52
#define TL_DOT 53


/*
	Sprite definitions
*/
#define SP_HAND_TL 0
#define SP_HAND_TR 1
#define SP_HAND_BL 2
#define SP_HAND_BR 3
#define SP_ARR_UP 4
#define SP_ARR_DN 5
#define SP_ARR_LF 6
#define SP_ARR_RT 7


/*
	The following are sound effects.

	Basically, all the registers set back to back for each sound.

	Sound registers were picked using sound.gb

	NOTE: these sounds are copied over from Fix It Felix Jr. fow now,
	till I can make some new ones just for Dots.
*/
#define SND_JUMP NR10_REG=0x66;NR11_REG=0x89;NR12_REG=0x46;NR13_REG=0xBE;NR14_REG=0x85;
#define SND_FALL NR10_REG=0x6E;NR11_REG=0x89;NR12_REG=0x46;NR13_REG=0xBE;NR14_REG=0x85;
#define SND_FIX NR10_REG=0x73;NR11_REG=0x95;NR12_REG=0x76;NR13_REG=0x78;NR14_REG=0x85;
#define SND_START NR10_REG=0x75;NR11_REG=0x68;NR12_REG=0x73;NR13_REG=0x7C;NR14_REG=0x86;
//#define SND_FIST NR41_REG=0x14;NR42_REG=0xA2;NR43_REG=0x79;NR44_REG=0xC0;
#define SND_FIST NR41_REG=0x14;NR42_REG=0x41;NR43_REG=0x79;NR44_REG=0xC0;
#define SND_STEP NR41_REG=0x14;NR42_REG=0x41;NR43_REG=0x71;NR44_REG=0xC0;

