/*
	game.c
	------

	This file houses the main logic and loop for the Dots game.

	Perhaps one day this code ought to be broken up into some clearer, logical,
	files. Maybe one for the pick-mode logic, one for the move-mode logic, and
	one for the results-mode logic.

	But as it stands, this file contains all the logic for the entire game,
	more-or-less. Due to the quirky nature of GBDK and LCC dev in C, this was
	written organically adding features as necessary. Refactoring would introduce
	the possibility of dealing with compiler quirks.

	If you're reading this, the main game code is all contained within this file,
	for all it's modes.

	Regarding the modes:

	MODE_PICK is when the cursor shows as a hand-pointer and the user can move
	the pointer around to "pick" a dot to start a chain.

	MODE_MOVE is the "selection" mode. Perhaps this could stand to be renamed to
	sometime like MODE_SELECT, but for now it's MODE_MOVE. Here the user moves
	to neighboring dots of the same color to build a chain.

	MODE_RESULTS is enabled when the time is up and the game moves to show the
	final score / best score results screen.

	--------------------------------------------------------------------------------

	Note about overall game-play implementation and flow:

	The main game code has two modes, MODE_PICK and MODE_MOVE.

	The game has an array of dots using color macros for their values.

	In MODE_PICK the controller allows a hand-pointer cursor to move around the main
	grid of dots, to "pick" a dot to start a chain on.

	When the A button is pressed and held, it switched to MODE_MOVE, to select a
	chain of similarily coloured neighboring dots.

	In MODE_MOVE the controller allows the cursor to move to valid neighbors to
	build the chain.

	After a chain is built, those dots are cleared from the grid / screen, dots above
	them begin falling down, and score points are awarded for the number of dots cleared.

	The game returns to MODE_PICK, and the pattern continues until the time runs out.

	The specifics are detailed in the code comments below.
*/


// game modes:
#define MODE_PICK 0
#define MODE_MOVE 1
#define MODE_RESULTS 2

// directions to move
#define UP 0
#define DOWN 1
#define LEFT 2
#define RIGHT 3

// the indicies our of falling boi's data (see below)
#define FALLING_ACTIVE 0
#define FALLING_COLOR 1
#define FALLING_COLUMN 2
#define FALLING_ROW 3


// function declartions to avoid implicit function declaration compile warnings
int getDigit(int number, int digit);
void setBKGTile(int x, int y, unsigned char tile, int attr);
int getBKGTile(BYTE x, BYTE y);
void Game_InitSprites();
void Game_InitGrid();
void Game_Start();
void Game_Loop();
void Game_UpdateSprites();
void Game_Input();
void Game_MoveCursor(int, int);
void Game_UpdateTime();
void Game_UpdateScore();
void Game_CheckValidPick();
void Game_ModeSwitch();
UBYTE Game_CheckMove(int x, int y);
void Game_CheckValidMoves();
void Game_ShrinkDot(int x, int y);
void Game_EmbiggenDot(int x, int y);
void Game_MoveSelection(int direction);
void Game_ClearAllLines();
void Game_UpdateGrid(int forceUpdate);
void Game_ClearSelectedDots();
void Game_SuperClear();
void Game_CheckEmptyDotGaps();
void Game_UpdateFallingDots();
void Game_SpawnFallingDot(int color, int x, int y);
void Game_CheckEmptyTops();
void Game_ResultsLoop();
void Game_SetBGColor(int color);
void Game_DrawNumber(int x, int y, int number);
void Game_PausedLoop();

// game variables:

// which mode the game is currently in, via the above macros: MODE_PICK, MODE_MOVE, or MODE_RESULTS
int gameMode;

// the time left in the game, in seconds
int time;

// the players current score
int score;

// the cursor's sprite position on screen
UBYTE cursorX;
UBYTE cursorY;

// the logical position cursor is on the grid
int cursorGridX;
int cursorGridY;

// joystick reading from last frame to compare for differences
int joy_last;

/*
	Since I'm not using any kind of sane "time" library, the seconds are measured in iterations
	of the games main loop, roughly estimating the length of seconds.

	The variable below is a counter for loops until we should subtract a second from our game time
*/
int secondsCounter;

/*
	current move color. When a dot is picked, the game switched to MODE_MOVE and only neighboring
	dots of this color (moveColor) can be moved to.
*/
char moveColor;

/*
	When a dot is selected while forming a chain of dots, we need to keep track of all the dots
	selected thus far. When the selection is ended, we can clear all the dots and score points
	for them.

	Below is a 2D array of selected dots.

	30 total entries, because our dot grid is 6x5 - that is, 5 rows of  dots, or 30 total dots.
	This array doesn't need to be bigger than 30, because that implies every dot on screen is
	selected.

	The second dimision of the array is size 2, which is used to store the (x, y) position
	of a selected dot. Here we define and initialize the dot selection list to all 0's.
*/
char selectedDots[30][2] = 
{
	{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0},
	{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0},
	{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0},
	{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0},
	{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0},
	{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}
};

/*
	When we enter dot-select mode, and add dots to our chain, this index keeps track both
	of our selected dot count (zero-based) and which slot to fill for future dots as we select them
*/
int selectionIndex = 0;

/*
	In the real DOT's game, when a square or rectangle of dots is formed of any-size, it enables a mode
	to clear all dots on the screen.

	But because I'm critically low on ROM space, to semi-mimmick this feature, this mode will be enabled
	whenever the dots selection path loops on itself.

	Realistically, it's pretty rare that a configuration of dots would ever spawn that would allow you
	to loop the dot selection chain on itself that ISN'T in a square.

	So, this will be a known issue - in my GameBoy Color dots game, instead of square/rectange detection,
	simply looping your dot chain line on itself will enable this mode.

	Perhaps one day if I decide to implement bank switching or optimize the code further I can afford to
	do proper square/rectangle detection. But for now - we'll just track loops on ourself.

	The variable below is 0 by default. But if the chain of dots crosses it's own path, it will be set
	to the `selectionIndex` when the cross happened. Thus, if `selectionIndex` continues to increase
	it will still be active.

	But we'll check if `selectionIndex` decreases beyond `superClearMode` and if so, re-evaluate the mode,
	possibly disabling it.

	If MODE_MOVE is exited whilst superClearMode!=0 then it will clear all dots of the selected colour, etc.
*/
int superClearMode;

// don't update grid every game loop, so use this counter to only update the grid after a number of loops
int gridUpdateLimiter = 0;

// don't update falling dots game loop, so use this counter to only update the falliong dots after a number of loops
int fallUpdateLimiter = 0;

// true when cursor is over a valid dot in pick-mode
UBYTE isOverValidDot;

// updated when cursor moves in MOVE mode
// represents valid move directions:
UBYTE canMoveUp = 0;
UBYTE canMoveDown = 0;
UBYTE canMoveLeft = 0;
UBYTE canMoveRight = 0;

// when moving or selecting a dot, it should animate to be big for a few frames
// if another dot is made to be big, it will cancel the current one
// this variable stores the current "big" dot grid location
UBYTE bigDotGridX = 0;
UBYTE bigDotGridY = 0;

// when a dot is made to be big, this timer will be set so it stays big for awhile
int bigDotTimer = 0;

/*
	Our game grid of dots is 6x5; that is, 6 columns and 5 rows.

	Below we will store the current state of the dots grid as a 2-dimension array of
	unsigned chars (to save on memory).

	Note that, the values in this grid are to be set / read with the corresponding macros:
	EMTPY, GREEN, BLUE, YELLOW, PURPLE, and RED.
*/
unsigned char grid[6][5];

// for now we will use a hard-coded default set of dots until random generation is implemented
unsigned char defaultGrid[6][5] = 
{
	{BLUE,		GREEN,		GREEN,		GREEN,		YELLOW,	},
	{BLUE,		PURPLE,		YELLOW,		RED, 		RED,	},
	{RED, 		GREEN, 		YELLOW,		PURPLE, 	RED,  	},
	{GREEN, 	GREEN,		GREEN, 		PURPLE, 	BLUE, 	},
	{BLUE, 		GREEN, 		GREEN,		BLUE,		BLUE,	},
	{BLUE, 		PURPLE, 	GREEN,		PURPLE,		BLUE,	}
};

/*
	Right, so, makin' 'em fall is gon' be a ticky son'a'bitch
	
	Basically, when dots are cleared we need to re-evaluate the board
	and make any dots that don't have a dot below them, fall.

	But to do so, that means we need to temporarily remove them from the grid
	then, animate them using a separate system.

	Once they "land" they need to be re-added to the grid for selection.

	Below we will have a multi-dimensional array, for a max of 30 falling dots.

	Since the grid is 6x5, there is a total of 30 possible falling dots at once.

	Rather than use dynamic memory or linked-lists, etc, for now we'll just
	use an array with the max possible falling dots.

	Because sorting is also expensive, everytime they falling dots are updated,
	the entire list of dots will be processed.

	So, the second dimension of the array will have the following fields:
		- active - 0 if this dot is no longer actively falling (i.e. it's old data)
		- color - the dot color to move
		- column - the x position of the dot that's falling
		- row - the current y position of the dot as it's falling
*/
unsigned char fallingBois[30][4] = {
	{0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0},
	{0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0},
	{0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,0,0,0}
};

/*
	Right, so, whenever Game_SpawnFallingDot(...) is called it will spawn a new falling dot.

	This means, until that dot "lands" we have falling dots to update / animate.
	
	Thus, Game_SpawnFallingDot(...) will set the variable below to 1 (true) since a new falling dot
	was spawned.

	In our Game_UpdateFallingDots() method, we will loop over all our fallingBois[30][4], and update
	any ones that are set active.

	However, if we don't find any active falling bois, we will set the variable below to 0, since
	an entire pass was completed with no active falling dots.

	This way, we won't have to loop over our 30 falling bois unless we have active ones.

	Perhaps in the future this could benefit from a dynamic memory solution, so we only have to
	loop over a dynamic list of active dots, and when that list shrinks to zero, no need for this
	variable below.

	But til' then, this shall do
*/
int gameHasFallingDots = 0;

/*
	When the user is moving / selecting dots to form a chain, lines must be drawn between the dots.

	However, the dots and lines are all drawn using the background grid and NOT sprites.
	(Because I didn't want to deal with a limited number of sprites and/or sprites interfering with
	the users pointer/move cursors sprites, etc)

	But when a dot is selected it's temporarily re-drawn as a "big" dot, which uses it's neighboring tiles.

	When the dot needs to shrink again, it has to replace it's neighboring tiles with the correct tile.

	If a line was added there, it should add a line instead of an empty tile.

	This grid represents the tiles on the screen, but only 0 or 1 if there's a line in that tile slot.

	NOTE: this is a huge waste of memory but makes the code easier to write. Perhaps in the future
	this array could be compressed to only care about tiles that could actually have lines on them.

	Oh well.

	Also, when making a selection, you shouldn't be able to over-lap your own chain, so this array
	will also be used for valid-move detition so you can't move on existing lines.
*/
char lines[20][18] =
{
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
};

// when showing results, slide the background grid down over time (e.g. slide screen down to results area):
int resultsSlide = 0;

// save best score on end-of-round
int bestScore = 0;


/**
 * Sets up the game to play
 */
void Game_Start()
{
	// due to limitation in GBDK and LCC, local vars always need to be defined on top
	int x;

	// clear our background
	Main_ClearBG();

	// use vram bank 1 for the various game-map attributes (i.e. pallete, horiz/vert flip)
	VBK_REG = 1;
	set_bkg_tiles(1, 1,  15, 1, time_and_score_attr);
	set_bkg_tiles(6, 19, 8,  2, results_attr);
	set_bkg_tiles(6, 22, 9,  1, score_attr);
	set_bkg_tiles(6, 25, 8,  1, best_attr);
	set_bkg_tiles(5, 29, 11, 1, press_start_attr);

	// use vram bank 0 for the various game-map tile ids
	VBK_REG = 0;
	set_bkg_tiles(1, 1,  15, 1, time_and_score);
	set_bkg_tiles(6, 19, 8,  2, results);
	set_bkg_tiles(6, 22, 6,  1, score_txt);
	set_bkg_tiles(6, 25, 5,  1, best);
	set_bkg_tiles(5, 29, 11, 1, press_start);

	// title screen always top left
	move_bkg(0, 0);

	// set up our game sprites
	Game_InitSprites();

	// set up our game grid
	Game_InitGrid();

	// no falling bois at start
	gameHasFallingDots = 0;
	for(x=0; x<30; x++)
		fallingBois[x][FALLING_ACTIVE] = 0;

	// (re)set game time and score
	time = 61;
	score = 0;

	// default cursor positioning is close to center
	cursorGridX = 2;
	cursorGridY = 1;
	cursorX = 23 + (cursorGridX*3*8);
	cursorY = 43 + (cursorGridY*3*8); 

	// default game mode to start is pick mode
	gameMode = MODE_PICK;

	// check if cursor is over a valid dot:
	Game_CheckValidPick();

	// assume no buttons pressed yet
	// when we call Game_Input() in our main loop, this will store
	// the value of the button from the last frame
	joy_last = 0;

	// start with 60 frames to count for our timer
	secondsCounter = 1;

	// no move colour yet
	moveColor = EMPTY;

	// update score and time on screen
	Game_UpdateScore();
	Game_UpdateTime();

	// make sure no left over lines from last round
	Game_ClearAllLines();

	// ready to show the game screen!
	DISPLAY_ON;

	// tells GBC we want to see sprites and the background, but not the "window"
	SHOW_SPRITES;
	SHOW_BKG;
	HIDE_WIN;

	// enter main game loop
	Game_Loop();

}// void Game_Start()


/**
 * Initialized the game's grid of dots with random dots or the default dots.
 *
 * If SELECT is held on the controller the default grid will load instead
 * of a randomly generated grid.
 */
void Game_InitGrid()
{
	// due to limitation in GBDK and LCC, local vars always need to be defined on top
	int i;
	int x;
	int y;
	UBYTE r;
	int randomDotColor;
	int color;
	int o;
	int selectHeldOnStart = 0;

	// if select was held on the start of this method, load default debug grid instead
	if(joypad() & J_SELECT)
		selectHeldOnStart=1;

	// reset grid
	for(x=0; x<6; x++)
	{
		for(y=0; y<5; y++)
		{
			// if SELECT is held, just load the default grid
			if(selectHeldOnStart==1)
				grid[x][y] = defaultGrid[x][y];

			// otherwise, pick random colours for the grid:
			else
			{
				// pick random colour as UBYTE, and modulo that mofo to be between 0 and 5, exclusive.
				r = rand();
				randomDotColor = (int)(r%5);
				
				// our colors start at 1, since 0 is empty.
				// so we'll add 1 to the result of the mod 5 above
				randomDotColor++;

				// set grid to random dot color
				grid[x][y] = randomDotColor;

			}// end if SELECT is held

		}// next y

	}// next x

	// SELECT is NOT held we should deliberately generate some random neighbors:
	if(!(selectHeldOnStart==1))
	{
		// loop pick 8 random dots to add similar neighbors too..
		for(i=0; i<8; i++)
		{
			/*
				So we pick a random dot in the middle of the grid...

				That is, not on the edges, so x-bounds 1-4 and y-bounds 1-3
			*/
			r = rand();
			x = (int)(r%4);
			x+=1;
			r = rand();
			y = (int)(r%3);
			y+=1;

			// get the color of the dot at this location
			color = grid[x][y];

			// pick a random neighbor to force to be the same colour.
			// this way, the starting map will always have a decent selection of combos off the bat
			r = rand();
			if(r%2==0)
			{
				r = rand();
				o = (int)(r%2);
				grid[x + ((o==0) ? -1 : 1)][y] = color;
			}else
			{
				r = rand();
				o = (int)(r%2);
				grid[x][y + ((o==0) ? -1 : 1)] = color;
			}
			
		}// next i

	}// endif SELECT not held

}// void Game_InitGrid()


/**
 * Sets up our sprites for use in the game
 *
 * For now, we will only have 8 total sprites:
 * 		- 4 for for the pointer hand graphic
 * 		- 4 for the Up, Down, Left, and Right move arrows
 */
void Game_InitSprites()
{
	// tell the GameBoy Color which tiles to use for the hand-pointer sprite slots.
	// the hand-pointer is composed out of four total sprites, in a 2x2 grid of 8x8 sprites
	set_sprite_tile(SP_HAND_TL, ST_HAND_TL);
	set_sprite_tile(SP_HAND_TR, ST_HAND_TR);
	set_sprite_tile(SP_HAND_BL, ST_HAND_BL);
	set_sprite_tile(SP_HAND_BR, ST_HAND_BR);

	// set the sprite pallets and attributes for our hand-pointer sprites
	set_sprite_prop(SP_HAND_TL, 0);
	set_sprite_prop(SP_HAND_TR, 0);
	set_sprite_prop(SP_HAND_BL, 0);
	set_sprite_prop(SP_HAND_BR, 0);

	// tell the GameBoy Color which tiles to use for the four move-direction arrow sprite slots.
	// the move arrows are composed out of two tiles, one horizontal and one vertical that will be flipped
	set_sprite_tile(SP_ARR_UP, ST_ARROW_VERT);
	set_sprite_tile(SP_ARR_DN, ST_ARROW_VERT);
	set_sprite_tile(SP_ARR_LF, ST_ARROW_HORIZ);
	set_sprite_tile(SP_ARR_RT, ST_ARROW_HORIZ);
	
	// set the sprite palletes and vertical/horizontal flip-bits for our four move-direction arrow sprites
	set_sprite_prop(SP_ARR_UP, S_FLIPY);
	set_sprite_prop(SP_ARR_DN, 0);
	set_sprite_prop(SP_ARR_LF, 0);
	set_sprite_prop(SP_ARR_RT, S_FLIPX);

}// void Game_InitSprites()


/**
 * Changes the background color of the main gameboy tile grid background
 *
 * @param color the index of the color, can use #defines EMPTY, GREEN, BLUE, YELLOW, PURPLE and RED
 */
void Game_SetBGColor(int color){
	
	// due to limitations of GBDK and LCC local vars always must be declared at the top of functions
	int i;
	int palettePos;

	// loop to set the first color in our palette array
	for(i=0; i<8; i++)
	{
		/*
			Because the palette array is single dimensional, despite being 8 pallets
			of four colors, we need to calculate the colors offset in this array
		*/
		palettePos = i*4;

		// set the new color:
		bgTilePalettes[palettePos] = BG_COLORS[color]; 
	
	}// next i

	// tell the GBC about our new palette colors with the background color swapped in
	set_bkg_palette(0, 8, bgTilePalettes);

}// void Game_SetBGColor(int color){


/**
 * Sets a BKG tile and it's attributes
 *
 * This is mostly a convienence function, as a wrapper to GBDK's set_bkg_tiles(...) method,
 * which requires two VRAM bank switches to complete the job, and some specific formatted params
 *
 * @param x the bg tile x position to set
 * @param y the bg tile y position to set
 * @param tile the integer of the tile index to set
 * @param attr the tiles attributes to set
 */
void setBKGTile(int x, int y, unsigned char tile, int attr)
{
	// set the tiles attributes (pallet, flips, etc.) in VRAM bank 1
	VBK_REG = 1;
	set_bkg_tiles(x, y, 1, 1, &attr);

	// set the tiles tile-ID in VRAM bank 0
	VBK_REG = 0;
	set_bkg_tiles(x, y, 1, 1, &tile);

}// void setBKTile(int x, int y, int tile, int attr)


/**
 * Returns the tile index for a tile on the background
 *
 * This is basically a conveinence function as a wrapper to GBDK's get_bkg_tiles(...) method,
 * which requires specific parameters and "out" style pointers instead of a return value.
 *
 * note: this method can be expensive / slow! Use sparingly!
 *
 * @param x the bg tile x position to get the tile-ID of
 * @param y the bg tile y position to get the tile-ID of
 * @returns the tile-ID index of the tile on the gameboy background grid
 */
int getBKGTile(BYTE x, BYTE y)
{
	// due to limitations of GBDK and LCC local vars always must be declared at the top of functions
	unsigned char tileData;

	/*
		Right, so, GBDK's get_bkg_tiles(...) method doesn't actually return anything.

		Instead it uses it's 5th parameter kinda like an "out" variable. In reality, it's just
		a pointer to an unsigned char buffer.

		Because `get_bkg_tiles(...)` is expected to return an array of tile-ID's, it expects
		it's 5th parameter to be a pointer to a buffer of unsigned char's.

		But because we're only sampling a 1x1 area of the background tiles, we only really need
		an array of size 1 (i.e. `unsigned char data[1];`).

		Since we will only get one result, `tileData` is defined as a regular variable above,
		not an array of size one. We pass in `&tileData` which will be populated as-if it was
		an array. But since we will only get one item back this won't cause a buffer-overrun error.

		Of course, since it's being used like an "out" variable, tileData needs to be set to something
		initially. Otherwise, the compiler will spit out a warning about the use of an uninitialized variable.
		We'll just set it to 0, knowing it will be over-written with the result of our `get_bkg_tiles(...)`
		request.
	*/
	tileData = 0;
	get_bkg_tiles(x, y, 1, 1, &tileData);

	// return tile data
	return tileData;

}// int getBKGTile(int x, int y);


/**
 * Makes a big dot at grid location X/Y back to normal size immediately
 *
 * When a dot is selected in MODE_PICK or moved to in MODE_MOVE it's temporarily made larger.
 * This is done by changing a serries of background tiles around the dots grid location to make
 * it appear larger.
 *
 * This method removes those tiles and sets the dot's neighboring background tiles to be emtpy,
 * or line-segment tiles where necessary.
 *
 * @param x the x grid position for the dot to shrink
 * @param y the y grid position for the dot to shrink
 */
void Game_ShrinkDot(int x, int y)
{
	// due to limitations of GBDK and LCC local vars always must be declared at the top of functions
	unsigned char dotColor;
	int bkgX;
	int bkgY;

	// get the color of the dot to embiggen
	dotColor = grid[x][y];

	/*
		Below we convert the logical grid X/Y to the gameboys background tile X/Y.
		The first grid dot, at (0, 0) is background tile (2, 3)
		Each grid position is a multiple of 3 background tiles away from (2, 3), etc
	*/
	bkgX = 2 + (x*3);
	bkgY = 3 + (y*3);

	/*
		Below we set the neighboring tiles to TL_EMPTY around the dot, and set the dot's center
		to the regular TL_DOT tile.

		However, if the user has a selection with lines drawn on the screen, we will use the
		`lines[20][18]` buffer to check if there is a line neighboring the dot.

		The `setBKGTile(...)` code below uses ternarys to set either TL_EMPTY or the appropriate
		TL_LINE_VERT/TL_LINE_HORIZ if there is a line neighboring the dot on screen.
	*/

	// top row:
	setBKGTile(bkgX-1,	bkgY-1,	TL_EMPTY, 	dotColor);
	setBKGTile(bkgX,	bkgY-1,	(lines[bkgX][bkgY-1]==1) ? TL_LINE_VERT : TL_EMPTY, 	dotColor);
	setBKGTile(bkgX+1,	bkgY-1,	TL_EMPTY, 	dotColor);
	
	// middle row
	setBKGTile(bkgX-1, 	bkgY, 	(lines[bkgX-1][bkgY]==1) ? TL_LINE_HORIZ : TL_EMPTY, 	dotColor);
	setBKGTile(bkgX, 	bkgY, 	TL_DOT, 	dotColor);
	setBKGTile(bkgX+1, 	bkgY, 	(lines[bkgX+1][bkgY]==1) ? TL_LINE_HORIZ : TL_EMPTY, 	dotColor);

	// bottom row:
	setBKGTile(bkgX-1,	bkgY+1,	TL_EMPTY, 	dotColor);
	setBKGTile(bkgX,	bkgY+1,	(lines[bkgX][bkgY+1]==1) ? TL_LINE_VERT : TL_EMPTY, 	dotColor);
	setBKGTile(bkgX+1,	bkgY+1,	TL_EMPTY, 	dotColor);

}// void Game_ShrinkDot(int x, int y)


/**
 * Makes a dot at grid location X/Y big for awhile
 *
 * When a dot is selected in MODE_PICK or moved to in MODE_MOVE it's temporarily made larger.
 * This is done by changing a serries of background tiles around the dots grid location to make
 * it appear larger.
 *
 * This method adds those tiles and sets the dot's neighboring background tiles to be a solid square.
 *
 * @param x the x grid position for the dot to embiggen
 * @param y the y grid position for the dot to embiggen
 */
void Game_EmbiggenDot(int x, int y)
{
	// due to limitations of GBDK and LCC local vars always must be declared at the top of functions
	unsigned char dotColor;
	int bkgX;
	int bkgY;

	// if we already have an embiggened dot that hasn't shrunken yet, shrink it first:
	if(bigDotTimer!=0)
		Game_ShrinkDot(bigDotGridX, bigDotGridY);

	// save the newly embiggened dot's location, so we know which dot to shrink when time's up
	bigDotGridX = x;
	bigDotGridY = y;

	// set new timer for this new big dot
	// note that this time is based on game-loop interation counts, not any normal measure of time
	bigDotTimer = 40;

	// get the dot color to embiggen
	dotColor = grid[x][y];

	/*
		Here we convert the logical grid X/Y to the gameboys background tile X/Y.
		The first grid dot, at (0, 0) is background tile (2, 3)
		Each grid position is a multiple of 3 background tiles away from (2, 3), etc
	*/
	bkgX = 2 + (x*3);
	bkgY = 3 + (y*3);

	// set all it's surrounding tiles to the appropriate big circle tiles

	// top row:
	setBKGTile(bkgX-1,	bkgY-1,	TL_DOT_BIG_CORNER, 			dotColor);
	setBKGTile(bkgX,	bkgY-1,	TL_DOT_BIG_TOP_AND_BOTTOM, 	dotColor);
	setBKGTile(bkgX+1,	bkgY-1,	TL_DOT_BIG_CORNER, 			dotColor | S_FLIPX);
	
	// middle row
	setBKGTile(bkgX-1, 	bkgY, 	TL_DOT_BIG_LEFT_AND_RIGHT, 	dotColor);
	setBKGTile(bkgX, 	bkgY, 	TL_DOT_BIG_CENTER, 			dotColor);
	setBKGTile(bkgX+1, 	bkgY, 	TL_DOT_BIG_LEFT_AND_RIGHT, 	dotColor | S_FLIPX);

	// bottom row:
	setBKGTile(bkgX-1,	bkgY+1,	TL_DOT_BIG_CORNER, 			dotColor | S_FLIPY);
	setBKGTile(bkgX,	bkgY+1,	TL_DOT_BIG_TOP_AND_BOTTOM, 	dotColor | S_FLIPY);
	setBKGTile(bkgX+1,	bkgY+1,	TL_DOT_BIG_CORNER, 			dotColor | S_FLIPY | S_FLIPX);

}// void Game_EmbiggenDot(int x, int y)


/**
 * Updates positions and visibility of sprites as appropriate for the game state
 */
void Game_UpdateSprites()
{
	// due to limitations of GBDK and LCC local vars always must be declared at the top of functions

	// the position the hand cursor should move towards
	int targetX;
	int targetY;

	// the difference between the hand cursors position and it's target position
	int deltaX;
	int deltaY;

	// depending on game mode, move / show correct sprites:

	// if we're in MODE_PICK we should show the four sprite's that make up the pointer-hand
	if(gameMode==MODE_PICK)
	{
		// calculate the cursor's screen-position in pixels for hand-pointer
		targetX = 23 + (cursorGridX*3*8);
		targetY = 43 + (cursorGridY*3*8); 

		// calculate how far-off the hand-pointer cursor's sprite position is from our target
		deltaX = targetX - cursorX;
		deltaY = targetY - cursorY;

		/*
			Because we're calculating our cursors target-position and delta towards that position,
			(basically a Vector2 delta), we will use this to move the hand-pointer sprites TOWARDS
			their target position.

			This is a basic for of animating the hand instead of just making it jump to it's
			new position in one frame.

			Below, if we have a delta on either axis, insted of moving the hand-pointer cursor sprites
			directlly, we will just move them one pixel towards their target.

			Eventually they will meet their target position, after a number of game-loops.
		*/
		if(deltaX!=0)
			cursorX += (deltaX<0) ? -1 : 1;
		if(deltaY!=0)
			cursorY += (deltaY<0) ? -1 : 1;
		
		// move hand-pointer cursor sprites into their new place
		move_sprite(SP_HAND_TL, cursorX, 	cursorY 	);
		move_sprite(SP_HAND_TR, cursorX+8, 	cursorY 	);
		move_sprite(SP_HAND_BL, cursorX, 	cursorY+8	);
		move_sprite(SP_HAND_BR, cursorX+8, 	cursorY+8	);
		
		/*
			Below we hide arrow sprites off screen.
			
			NOTE: because sprites are 8x8 pixel tiles, we can "hide" them by setting them
			(-8, -8) pixels off screen to the top right.

			While there MAY be a GBDK/ GameBoy Color method to disable sprites, I'm not sure of it.

			So to hide the arrow sprites below, we'll just put them off screen where they can't be seen,
			as long as we'lre in this mode.
		*/
		move_sprite(SP_ARR_UP, -8, 	-8 	);
		move_sprite(SP_ARR_DN, -8, 	-8 	);
		move_sprite(SP_ARR_LF, -8, 	-8	);
		move_sprite(SP_ARR_RT, -8, 	-8	);

	// otherwise, if we're in MODE_MOVE we should show just the appropriate move-arrow sprites,
	// and hide the hand-pointer cursor.
	}else if(gameMode==MODE_MOVE)
	{
		/*
			Below we calculate cursor position for the center point of the arrows.

			Note that, unlike the hand-pointer cursor which is a 2x2 grid of sprites
			to comprise it's full shape, the arrows need to appear independently of each other,
			to the left / right and above / below the selected dot.

			So here we caculate the "center" position of the arrows, i.e. the location of the
			selected dot.
		*/
		cursorX = 24 + (cursorGridX*3*8);
		cursorY = 40 + (cursorGridY*3*8); 

		/*
			When in MODE_MOVE the cursor can only move in the direction of valid neighbors.

			Thus, before we move/show our arrow sprites, we call `Game_CheckValidMoves()`
			to determine which arrows to currently show.
		*/
		Game_CheckValidMoves();

		/*
			The aforementioned `Game_CheckValidMoves()` method will update the global variables:
				- canMoveLeft
				- canMoveRight
				- canMoveUp
				- canMoveDown

			Which represent if that move direction is currently valid given the selected dot's
			grid position and it's neighbors.

			Based on which directions are allowed, we should show/move those arrows into position,
			or hide them off screen at (-8, -8) pixels if that direction is invalid.
		*/
		if(canMoveLeft==0)
			move_sprite(SP_ARR_LF, -8, 	-8 	);
		else
			move_sprite(SP_ARR_LF, cursorX-8, 	cursorY		);

		if(canMoveRight==0)
			move_sprite(SP_ARR_RT, -8, 	-8 	);
		else
			move_sprite(SP_ARR_RT, cursorX+8, 	cursorY		);

		if(canMoveUp==0)
			move_sprite(SP_ARR_UP, -8, 	-8 	);
		else
			move_sprite(SP_ARR_UP, cursorX, 	cursorY-8 	);

		if(canMoveDown==0)
			move_sprite(SP_ARR_DN, -8, 	-8 	);
		else
			move_sprite(SP_ARR_DN, cursorX, 	cursorY+8 	);
		
		// since we're in MODE_MOVE (aka chain selection mode), we shouldn't show any of the hand-pointer
		// cursor sprites at all. Let's hide them by putting them off screen at (-8, -8) pixels.
		move_sprite(SP_HAND_TL, -8, 	-8 	);
		move_sprite(SP_HAND_TR, -8, 	-8 	);
		move_sprite(SP_HAND_BL, -8, 	-8	);
		move_sprite(SP_HAND_BR, -8, 	-8	);

	}// end if

	/*
		When in MODE_PICK sometimes the hand-pointer cursor is over a dot that has no
		similarily-coloured neighbors.

		This dot is an "invalid" dot to select, because selecting it wouldn't allow you to
		make any connections to neighbors.

		In the event that the MODE_PICK hand-pointer cursor is over an invalid-dot, we will
		change the cursor's sprite palette to make it yellow-orange intead of gray-scale.

		In this case, I'm just re-using one of the other existing sprite-palettes for the
		yellow dots.

		Below, we update the hand-poitner cursor sprite's GBC palettes depending on if it's
		over a valid dot to select or not.
	*/
	if(isOverValidDot==1)
	{
		// since the cursor is over a valid-dot to select, set the hand-pointer cursor sprite's
		// GBC palettes to be 0, aka my default gray-scale palette
		set_sprite_prop(SP_HAND_TL, 0);
		set_sprite_prop(SP_HAND_TR, 0);
		set_sprite_prop(SP_HAND_BL, 0);
		set_sprite_prop(SP_HAND_BR, 0);
	}else
	{
		// since the cursor is over an invalid-dot to select, set the hand-pointer cursor sprite's
		// GBC palettes to be 3, the yellow-orange palette
		set_sprite_prop(SP_HAND_TL, 3);
		set_sprite_prop(SP_HAND_TR, 3);
		set_sprite_prop(SP_HAND_BL, 3);
		set_sprite_prop(SP_HAND_BR, 3);
	}

}// void Game_UpdateSprites()


/**
 * Implements a hackish way to get the digit in an Integers specific column.
 *
 * @param number the number to get a digit column from
 * @param digit the colum to return the digit from
 * @returns an interger that is the digit in the chosen column
 */
int getDigit(int number, int digit)
{
	// if we want the first digit ("1's column"), just return modulo 10
	if(digit==1)
	{
		return number % 10;

	// if we want the second digit ("10's column"), devide the modulo 100 by 10 and return
	}else if(digit==2)
	{
		return (number % 100) / 10;

	// if we want the third digit ("100's column"), devide the modulo 1000 by 100 and return
	}else if(digit==3)
	{
		return (number % 1000) / 100;
	}

}// int getDigit(int number, int digit)


/**
 * Draws a number on the GameBoys background tile grid at the specified location, (x, y).
 *
 * @param x the x position on the GameBoy's background tile grid to start drawing the number
 * @param y the y position on the GameBoy's background tile grid to start drawing the number
 * @param number the number to draw with tiles on the GameBoy's background tile grid
 */
void Game_DrawNumber(int x, int y, int number)
{
	// if we have more than 2 digits in our number, set all 3 tiles
	if(number>=100)
	{
		set_bkg_tiles( x, 	y, 1, 1, &TL_DIGITS[getDigit(number, 3)]);
		set_bkg_tiles( x+1, y, 1, 1, &TL_DIGITS[getDigit(number, 2)]);
		set_bkg_tiles( x+2, y, 1, 1, &TL_DIGITS[getDigit(number, 1)]);

	// otherwise, if we have only 2 digits, set those tiles and an empty third tile
	}else if(number>=10)
	{
		set_bkg_tiles( x, 	y, 1, 1, &TL_DIGITS[getDigit(number, 2)]);
		set_bkg_tiles( x+1, y, 1, 1, &TL_DIGITS[getDigit(number, 1)]);
		set_bkg_tiles( x+2, y, 1, 1, &TL_NONE);

	// lastly, if we only have 1 digit, set it's tile and two empty tiles afterward.
	}else
	{
		set_bkg_tiles( x, 	y, 1, 1, &TL_DIGITS[number]);
		set_bkg_tiles( x+1, y, 1, 1, &TL_NONE);
		set_bkg_tiles( x+2, y, 1, 1, &TL_NONE);
	}

}// void Game_DrawNumber(int x, int y, int number)


/**
 * Updates the game clock, both locically and on screen
 */
void Game_UpdateTime()
{
	/*
		Because we're not using any reality based time-keeping system, we decrement seconds
		roughly based on a counter of our games main-loop.

		Once a specific amount of game-loops have iterated we can decrement our game-time.

		Below we decrement our game-loop counter.
	*/
	secondsCounter--;

	// if we hit 0, reset timer and decrease the actual game time:
	if(secondsCounter<=0)
	{
		// reset to 300 frames
		secondsCounter=300;

		// derease game time:
		time--;

		// if time ran out, we should end the MODE_PICK / MODE_MOVE gameplay modes
		// and move onto the end-of-game reults screen.
		if(time<=0)
		{
			// set game to the results-view mode and make sure it's slide-in starts at 0 pixels
			gameMode = MODE_RESULTS;
			resultsSlide = 0;
		
		}// end if time up

		// update the GameBoy's background grid tiles with the time number at it's correct location
		Game_DrawNumber(6, 1, time);

	}// end if secondsCounter<0

}// void Game_UpdateTime()


/**
 * Updates the score tiles on the screen
 */
void Game_UpdateScore()
{
	/*
		NOTE:

		Originally for debug I capped the score at 999 b/c to avoid code that
		generates more than 3 digits on scren.

		However, after play-testing I realized that it's EXTREMELY UNLIKELY that
		any player would ever break a 1K score in just 60 seconds.

		It might technically be possible, but it's an extreme edge-case.

		Because every byte of rom-space is precious, I've commented-out this check
		all together b/c I really need the space right now.

		This IS a glitch. If the user scores more than 999, they will see their score
		loop around to 00X, etc. However, b/c the value is over 100, the score
		display will still show "007" for instance, as opposed to just "7" if their
		score was actually just 7.

		The same is true for the results screen and the end-of-game.

		B/C the rom space is precious, I'm knowingly allowing this glitch to exist.

		Contrats to any players that can clear 1K in 60 seconds.
	*/
	// if(score>999)
	// 	score=999;

	// update the GameBoy's background grid tiles with the score number at it's correct location
	Game_DrawNumber(16, 1, score);

}// void Game_UpdateScore();


/**
 * Checks if the cursor is over a valid dot to pick, to start connecting
 */
void Game_CheckValidPick()
{
	// due to limitation in GBDK and LCC, local vars always need to be defined on top
	unsigned char cursorColor;
	unsigned char otherColor;
	int testX;
	int testY;

	// get color of the dot under the cursor:
	cursorColor = grid[cursorGridX][cursorGridY];

	// if it's over type "EMPTY" then it's NOT over a valid dot
	if(cursorColor==EMPTY)
	{
		isOverValidDot = 0;
		return;
	}

	// check if there's a same-color to the left on the grid:
	if(cursorGridX>0)
	{
		testX = cursorGridX-1;
		testY = cursorGridY;
		otherColor = (grid[testX][testY]);
		if(otherColor==cursorColor)
		{
			isOverValidDot = 1;
			return;
		}
	}

	// check if there's a same-color to the right on the grid:
	if(cursorGridX<5)
	{
		testX = cursorGridX+1;
		testY = cursorGridY;
		otherColor = (grid[testX][testY]);
		if(otherColor==cursorColor)
		{
			isOverValidDot = 1;
			return;
		}
	}

	// check if there's a same-color above on the grid:
	if(cursorGridY>=1)
	{
		testX = cursorGridX;
		testY = cursorGridY-1;
		otherColor = (grid[testX][testY]);
		if(otherColor==cursorColor)
		{
			isOverValidDot = 1;
			return;
		}
	}

	// check if there's a same-color below on the grid:
	if(cursorGridY<4)
	{
		testX = cursorGridX;
		testY = cursorGridY+1;
		otherColor = (grid[testX][testY]);
		if(otherColor==cursorColor)
		{
			isOverValidDot = 1;
			return;
		}
	}

	// if we got this far, it has no similar colour neighbors
	isOverValidDot = 0;

}// void Game_CheckValidPick()


/**
 * To possibly save on ROM size and for code-reuse, this method checks if a move direction is valid
 *
 * @param x the x offset from our cursorGridX to test for valid move
 * @param y the y offset from our cursorGridX to test for valid move
 * @returns either 1 or 0 if the move is valid based on current circumstances
 */
UBYTE Game_CheckMove(int x, int y)
{
	// due to limitation in GBDK and LCC, local vars always need to be defined on top
	int otherColor;
	int testX = cursorGridX + x;
	int testY = cursorGridY + y;
	int bkgX = (cursorGridX*3) + 2;
	int bkgY = (cursorGridY*3) + 3;

	// if either x or y is out-of-bounds, this is def not valid move:
	if(testX<0 || testX>=6 || testY<0 || testY>=5)
		return 0;

	// get color of neighboring dot to test
	otherColor = (grid[testX][testY]);

	// if color is mismatched to our current move color, no dice:
	if(otherColor!=moveColor)
		return 0;

	// if our testX/testY is the last selected position, allow the move
	if(selectionIndex>=1)
		if((selectedDots[selectionIndex-1][0]==testX) && (selectedDots[selectionIndex-1][1]==testY))
			return 1;

	// if there is already a "line" adjacent to this dot, don't allow move in that direction:
	if((lines[bkgX+x][bkgY+y])==1)
		return 0;

	// if we passed all the checks, return 1
	return 1;

}// UBYTE Game_CheckMove(int x, int y)

/*
 * Checks which moves are currently allowed in move mode
 */
void Game_CheckValidMoves()
{
	// check which directions are applicable to move in
	canMoveLeft 	= Game_CheckMove(-1,  0 );
	canMoveRight 	= Game_CheckMove( 1,  0 );
	canMoveUp 		= Game_CheckMove( 0, -1 );
	canMoveDown 	= Game_CheckMove( 0,  1 );

}// void Game_CheckValidMoves()


/**
 * Handles input from the controller and responds appropriately based on gameMode.
 *
 * As mentioned in the header comment of this file, this code can/could/should/would be
 * impleted across different files to handle logic for each game mode.
 *
 * But due to GBDK quirkyness, my willingness to refactor is 0 - so this method handles
 * controller input for both MODE_PICK and MODE_MOVE in one. Fun.
 */
void Game_Input()
{
	// in MODE_PICK the only specific input code pertains to moving the hand-pointer cursor
	if(gameMode==MODE_PICK)
	{
		// check direction buttons, and move cursor if D-PAD direction is pressed this frame
		// and not on the last frame (i.e. joy_last)
		if((joypad() & J_UP) && !(joy_last & J_UP))
			Game_MoveCursor(0, -1);
		if((joypad() & J_DOWN) && !(joy_last & J_DOWN))
			Game_MoveCursor(0, 1);
		if((joypad() & J_LEFT) && !(joy_last & J_LEFT))
			Game_MoveCursor(-1, 0);
		if((joypad() & J_RIGHT) && !(joy_last & J_RIGHT))
			Game_MoveCursor(1, 0);
	
	}// endif MODE_PICK

	// in MODE_MOVE the only specific input pertains to moving the selection arrows cursor
	if(gameMode==MODE_MOVE)
	{
		// check direction buttons, and move cursor if D-PAD direction is pressed this frame
		// and not on the last frame (i.e. joy_last)
		if((joypad() & J_UP) && !(joy_last & J_UP) && canMoveUp==1)
			Game_MoveSelection(UP);
		if((joypad() & J_DOWN) && !(joy_last & J_DOWN) && canMoveDown==1)
			Game_MoveSelection(DOWN);
		if((joypad() & J_LEFT) && !(joy_last & J_LEFT) && canMoveLeft==1)
			Game_MoveSelection(LEFT);
		if((joypad() & J_RIGHT) && !(joy_last & J_RIGHT) && canMoveRight==1)
			Game_MoveSelection(RIGHT);
	
	}// endif MODE_MOVE

	/*
		Right, so, the A button is used to pick a dot to form a chain in MODE_PICK,
		but it's release is also used in MODE_MOVE to end a chain selection.

		When in pick mode, we only want to enter MODE_MOVE if the A button was pressed,
		but previously wasn't pressed (held).

		Our `Game_ModeSwitch()` handles switching between MODE_MOVE and MODE_PICK in
		both directions.

		Thus, we say `((joypad() & J_A) && !(joy_last & J_A))` if A was pressed, but wasn't
		pressed on the last game-loop, then we should switch modes. This pertains to both
		MODE_PICK and MODE_MOVE (more on this below).

		However, when we're in MODE_MOVE we want the release of A to also trigger a mode switch
		(and therefore end the chain-building MODE_MOVE).

		Thus, we say `(!(joypad() & J_A) && gameMode==MODE_MOVE)`, meaning, if we're in MODE_MOVE
		and A is NOT PRESSED currently, then switch mode (i.e. exit MODE_MOVE).

		Because the controls are designed so that you hold A to select a dot, and then release A
		to end your chain, these two conditions are asymetric.

		Pressing A requires it to be a "fresh" press, such that A was previously not held down.

		Where as releasing A will cause a mode switch only if in MODE_MOVE.

		For reasons unbeknownst to me, sometimes you can enter MODE_MOVE and the release of A
		was already triggered so releasing A physically does nothing.

		In this case, the first condition `((joypad() & J_A) && !(joy_last & J_A))` allows you
		to press A again to exit the MODE_MOVE.

		This works 100% of the time 90% of the time. But sometimes it ... doesn't. It's a weird
		edge-case, but this two-part IF covers both cases with a second-A-press fall back in
		the event you're stuck in MODE_MOVE.

		It is what it is.
	*/
	if(
		((joypad() & J_A) && !(joy_last & J_A))
		||
		(!(joypad() & J_A) && gameMode==MODE_MOVE)
	  )
	{
		Game_ModeSwitch();

	}// endif A-pushed / A-released mode switch

	/*
		If the user presses B whilst in MODE_MOVE, it should cancel the chain selection.

		So below we test if B was pressed in MODE_MOVE, and if so, switch back to MODE_PICK,
		and also clear the lines drawn on the screen, if any.
	*/
	if((joypad() & J_B) && gameMode==MODE_MOVE)
	{
		// set mode back to pick. 
		gameMode = MODE_PICK;

		// Since we were on a valid dot-already, we don't need to recheck valid picks
		//Game_CheckValidPick();

		// clear any lines on screen
		Game_ClearAllLines();

	}// endif B canceled MODE_MOVE

	/*
		If start is pressed during either MODE_MOVE or MODE_PICK, we should temporarily enter the
		paused state
	*/
	if(joypad() & J_START)
		Game_PausedLoop();

	// save our buttons states for next frame
	joy_last = joypad();

}// void Game_Input()


/**
 * Shows pause screen and waits for start to unpause
 */
void Game_PausedLoop()
{
	//clear background
	Main_ClearBG();

	// show paused text by setting paused map to both VRAM buffers
	VBK_REG = 1;
	set_bkg_tiles(7, 8, 6, 1, paused_attr);
	VBK_REG = 0;
	set_bkg_tiles(7, 8, 6, 1, paused);

	// hide sprites during pause screen
	HIDE_SPRITES;

	// wait for start to be unpressed
	while(joypad() & J_START);

	// wait for start to be pressed again
	while(!(joypad() & J_START));

	// wait for start to be unpressed
	while(joypad() & J_START);

	//clear background
	Main_ClearBG();

	// show srpites again
	SHOW_SPRITES;

	// rebuild background with time and score text on both VRAM buffers
	VBK_REG = 1;
	set_bkg_tiles(1, 1,  15, 1, time_and_score_attr);
	VBK_REG = 0;
	set_bkg_tiles(1, 1,  15, 1, time_and_score);

	// call regular game gfx updates to redraw screen
	Game_DrawNumber(6, 1, time);
	Game_UpdateScore();
	Game_UpdateGrid(1);

}// Game_PausedLoop()


/**
 * Moves the dot chain selection in a direction, if applicable.
 *
 * This method also adds line tiles to our background tile tile, as well as line indicators
 * to our `lines[20][18]` array so we can determine if there's lines here in the future.
 *
 * @param direction the direction to move, using macros UP, DOWN, LEFT, and RIGHT
 */
void Game_MoveSelection(int direction)
{
	// due to limitation in GBDK and LCC, local vars always need to be defined on top
	int i;
	int xDir;
	int yDir;
	int gridStartX;
	int gridStartY;
	int lineTile = TL_EMPTY;
	int bkgStartX;
	int bkgStartY;
	char wentBack = 0;

	// save start grid position before we move
	gridStartX = cursorGridX;
	gridStartY = cursorGridY;

	// no movement by default:
	xDir = 0;
	yDir = 0;

	// use ternarys to determine move directions
	if(direction==LEFT || direction==RIGHT)
		xDir = (direction==LEFT) ? -1 : 1;
	if(direction==UP || direction==DOWN)
		yDir = (direction==UP) ? -1 : 1;
	
	// move cursor to new position
	Game_MoveCursor(xDir, yDir);

	/*
		H'okay, so, if we moved to the dot position that was the last selected dot in our
		array of selected dots, we should decrease the the dot selection index since we went back,
		but also we should delete the connection lines both from our logical lines[20][18] array
		and from the GameBoy's grid tiles
	*/
	if(selectionIndex>=1)
		if((selectedDots[selectionIndex-1][0]==cursorGridX) && (selectedDots[selectionIndex-1][1]==cursorGridY))
		{
			// decrease selection index since we went "back"
			selectionIndex--;

			// true since we wen't back
			wentBack = 1;

			// if we are lower than a previously set superClearMode, cancel the superClearMode
			if(selectionIndex<superClearMode)
				superClearMode=0;

		}// end if wen't back

	// since we're moving to a new dot, add it to our list of dots if we didn't "go back"
	if(wentBack==0)
	{
		selectionIndex++;
		selectedDots[selectionIndex][0] = cursorGridX;
		selectedDots[selectionIndex][1] = cursorGridY;
	}// endif not went back

	// temporarily make dot bigger
	Game_EmbiggenDot(cursorGridX, cursorGridY);

	// based on direction we're moving, add line tiles between dots

	// pick line tile to use based on direction, unless we went back, in which case use empty
	if(wentBack==0)
		lineTile = (direction==UP || direction==DOWN) ? TL_LINE_VERT : TL_LINE_HORIZ;

	// calculate background grid position based on our start tile:
	bkgStartX = 2 + (gridStartX*3);
	bkgStartY = 3 + (gridStartY*3);

	// set tiles based on direction:
	setBKGTile(bkgStartX+xDir, 		bkgStartY+yDir, 	lineTile, moveColor);
	setBKGTile(bkgStartX+(xDir*2), 	bkgStartY+(yDir*2), lineTile, moveColor);
	
	// set our logical line connections in the matrix
	lines[bkgStartX+xDir][bkgStartY+yDir] = 1-wentBack;
	lines[bkgStartX+(xDir*2)][bkgStartY+(yDir*2)] = 1-wentBack;

	/*
		Below we loop to check if we crossed our own path (see comment on `superClearMode` at the top of the file)
		If we DID cross our own path, set this selection index to be the super clear index.

		This way, if the super clear-index is greater than 0 we're in super-clear mode.

		So, below we'll check for a loop if we have at least 3 dots in our selection (the smallest possible "loop",
		or "square" is 4 dots in a row, so we need at least 3). But also, if we're already in superClearMode,
		no need to check again. (I.E. we should only check if superClearMode==0)
	*/
	if(selectionIndex>3 && superClearMode==0)
		for(i=0; i<(selectionIndex); i++)
		{
			// check if the current dot already exists in list, if so, enable superClearMode
			if((selectedDots[i][0]==selectedDots[selectionIndex][0]) && (selectedDots[i][1]==selectedDots[selectionIndex][1]))
			{
				// save index of where we crossed this previous dot:
				superClearMode=selectionIndex;

				// GTFO since we found a cross
				break;

			}// end if found dot chain path cross

		}// next i

	// set background colur based on superClearMode
	Game_SetBGColor((superClearMode==0) ? EMPTY : moveColor);

}// void Game_MoveSelection(int direction)


/**
 * Clears all lines drawn on gameboard.
 *
 * When MODE_MOVE is exited, either due to completing a chain or canceling MODE_MODE,
 * any line tiles that were added to the background tile grid should be cleared to empty tiles.
 * This method does just that.
 */
void Game_ClearAllLines()
{
	// due to limitation in GBDK and LCC, local vars always need to be defined on top
	int x;
	int y;

	// clear all our line matrix lines
	for(x=0; x<20; x++)
	{
		for(y=0; y<18; y++)
		{
			lines[x][y] = 0;
		}// next y
	}// next x

	// clear all gaps between dots on rows in our background tile grid
	for(y=3; y<16; y+=3)
	{
		for(x=2; x<15; x+=3)
		{
			setBKGTile(x+1, y, TL_EMPTY, moveColor);
			setBKGTile(x+2, y, TL_EMPTY, moveColor);
		}// next x
	}// next y

	// clear all gaps between dos on coloumns in our background tile grid
	for(x=2; x<18; x+=3)
	{
		for(y=3; y<13; y+=3)
		{
			setBKGTile(x, y+1, TL_EMPTY, moveColor);
			setBKGTile(x, y+2, TL_EMPTY, moveColor);
		}// next y
	}// next x

}// void Game_ClearAllLines()


/**
 * Delete all dots of move-color when superClearMode is enabled at the end of a `Game_ModeSwitch()`
 *
 * When MODE_MOVE ends and superClearMode is a non-zero value, this method is called to delete all
 * dots of this the selection colour.
 */
void Game_SuperClear()
{
	// due to limitation in GBDK and LCC, local vars always need to be defined on top
	int x;
	int y;

	/*
		h'okay, so, since we ended the MODE_MOVE with superClearMode enabled, we don't really care
		about the actual dot's selection.

		So instead, we should rebuild the selection with all the dots of the current moveColor
		(because superClearMode removes all dot's of said colour).

		So, we'll reset the selectedIndex to 0.

		Then loop over the entire dots grid playfield, and an dots that match our colour will be
		added to our selection list.

		When this method exits, it will return to the `Game_ModeSwitch()` code to tally the score
		and remove all the selected dots.

		B/C we're editing our selection to include all dots of this colour, the score will update
		appropriately and all these dots will be cleared, etc.
	*/

	// As noted above, reset selection index since we're rebuilding the selection
	selectionIndex = 0;

	// loop over dot grid and "select" all dots of our current moveColor
	for(x=0; x<6; x++)
	{
		for(y=0; y<5; y++)
		{
			// if the grids colour matches our move colour, add it to our selection:
			if(grid[x][y]==moveColor)
			{
				// add dot to our selection
				selectedDots[selectionIndex][0] = x;
				selectedDots[selectionIndex][1] = y;
				
				// and increment our selection index
				selectionIndex++;

			}// end if grid matches move colour

		}// next y

	}// next x

	// since we will have over-incremented the selection index by 1, remove the 1
	selectionIndex--;

}// void Game_SuperClear()


/**
 * When A is pushed or released, we should possibly switch mode based on the current state
 *
 * If the game is in MODE_PICK and A is PUSHED we should select the dot and enter MODE_MOVE.
 * If the game is in MODE_MOVE and A is RELEASED (or pushed), we should clear the selected
 * dot chain (if any), and switch back to MODE_PICK.
 */
void Game_ModeSwitch()
{
	/*
		If we're currently in MODE_PICK we should select the dot under the hand-pointer cursor to
		start a new selection chain of dots.

		Or, if the dot under the hand-pointer cursor is invalid (it doesn't have similarily coloured
		neighbors), then we should just play an arrow sound and GTFO.
	*/
	if(gameMode==MODE_PICK)
	{
		// if we're over an invalid selection, just play error sound and GTFO
		if(isOverValidDot==0)
		{
			SND_FALL
			return;

		}// endif over invalid dot

		// get the colour of the dot the hand-pointer cursor is over:
		moveColor = grid[cursorGridX][cursorGridY];

		/*
			Below we set thje arrow sprite colors to match that of the selected dot's color (moveColor)
			Note that, we need to use bitwise | operators for the UP and RIGHT arrows to keep their
			horizontal / vertical sprite flip attributes intact.
		*/
		set_sprite_prop(SP_ARR_UP, moveColor | S_FLIPY);
		set_sprite_prop(SP_ARR_DN, moveColor);
		set_sprite_prop(SP_ARR_LF, moveColor);
		set_sprite_prop(SP_ARR_RT, moveColor | S_FLIPX);

		/*
			Since we're begininng a new dot-chain selection in MODE_MOVE, the index should to reset
			to 0, and the 0th selection dot should be set to the dot our hand-pointer cursor is over.
		*/
		selectionIndex = 0;
		selectedDots[0][0] = cursorGridX;
		selectedDots[0][1] = cursorGridY;

		// no super clear mode by default, so set it to 0
		superClearMode = 0;

		// make selected dot temporarily a big boi
		Game_EmbiggenDot(cursorGridX, cursorGridY);

		// play dot select / move sound
		SND_JUMP

		// switch mode to MOVE
		gameMode = MODE_MOVE;
		return;

	}// end if MODE_PICK

	/*
		If we're currently in MODE_MOVE, we must switch out to MODE_PICK.

		However, if we have a chain of dots bigger than 1 (i.e. selectionIndex is > 0),
		we should clear those dots from the grid and start dots above their gaps to
		be falling down.
	*/
	if(gameMode==MODE_MOVE)
	{
		// if we have more than one dot selected, clear dots from grid and add points to score
		if(selectionIndex!=0)
		{
			/*
				If super clear mode was enabled, we should rebuild our list of selected dots to
				include all dots of this colour on the screen.

				Since this will also affect the score / selectionIndex, this need to be re-calculted
				as well.
			*/
			if(superClearMode!=0)
				Game_SuperClear();

			// add a point to our score for every dot in this chain.
			// Since selectionIndex is 0-based, we can use that +1 to calculate the score.
			score += (selectionIndex+1);

			// update the score tiles on the screen
			Game_UpdateScore();

			// remove selected dots from game board, and make dots above fall, if any
			Game_ClearSelectedDots();

			// reset background if super clear was enabled
			Game_SetBGColor(EMPTY);

			// play score sound
			SND_FIX
		
		}// endif selection has more than one dot

		// set game mode back to pick
		gameMode = MODE_PICK;

		// clean up the display by removing connection lines (if any), and hiding deleted dots
		Game_ClearAllLines();
		Game_UpdateGrid(1);

		// since grid state may have changed, the cursor might be over an invalid dot now
		Game_CheckValidPick();
	
	}// end if MODE_MOVE

}// void Game_ModeSwitch()


/**
 * Clears all dots from previous move / select mode.
 *
 * When the game is in MODE_MOVE, neighboring dots are selected / chained.
 * If MODE_MOVE is exited with a chain length greater than 1, those dots are deleted,
 * and a point is awarded to the score for every dot in the chain.
 *
 * This method deletes those dots from our grid, leaving empty holes in the grid of
 * dots, which will eventually be filled in by dots falling from above.
 */
void Game_ClearSelectedDots()
{
	// due to limitation in GBDK and LCC, local vars always need to be defined on top
	int i;

	// Remove the dots in our array of selectedDots[30][2] from our grid dot array
	for(i=0; i<=selectionIndex; i++)
		grid[selectedDots[i][0]][selectedDots[i][1]] = EMPTY;

	// now that dot's have been removed, check the empty dot gaps to start spawning falling dots
	Game_CheckEmptyDotGaps();

	/*
		After calling the above `Game_CheckEmptyDotGaps()` method, the global variable:
		`gameHasFallingDots` will be set to `1` if even a single dot has begun falling to
		fill in a gap.

		However, in the case that only dots on the top-row were cleared (or a column of dot
		leading to the top row was cleared), there will be no dots above the empty holes to "fall".

		This will lead to a case where there are empty holes on the game grid, but no dots to
		fall into place.

		Our method `Game_CheckEmptyTops()` accounts for this. If there are no currently falling dots,
		but there are empty holes on the top row, this method spawns completely new dots to fall in,
		and fill those gaps.

		Note that, if we DID find dots to fall, `Game_CheckEmptyDots()` would be called eventually
		anyway, when those dots settled into place (i.e. "land" in the grid).

		Whether here, or in `Game_UpdateFalling()` the method `Game_CheckEmptyTops()` will be called
		one way or another to make sure no gaps remain on to the top (first row) of the grid playfield.
	*/
	if(gameHasFallingDots==0)
		Game_CheckEmptyTops();

}// void Game_ClearSelectedDots()

/**
 * Loops over the gameboard and checks if any there's any empty gaps.
 *
 * If we find a gap in a column on the dots-grid play field, any dots above that gap
 * should be set to be "falling dots".
 */
void Game_CheckEmptyDotGaps()
{
	// due to limitation in GBDK and LCC, local vars always need to be defined on top
	int i;
	int c;
	BYTE r;

	/*
		Right, so now for some tricky annoying shit.

		To make dots fall, we need to loop over each column from the bottom up and check for holes.

		If we find a hole, we need to set ALL the dots above that hole to be falling.
		This means, we need to remove the dots from our grid, since they can't be selected whilst falling.
		We also need to spawn a new falling dot of the same color and position as the dot we cleared.

		The falling code will handle them landing later...
	*/

	// loop over each column to check for holes
	for(c=0; c<6; c++)
	{
		// from the bottom-up, loop over the rows in the column
		for(r=4; r>=0; r--)
		{
			// if the grid has a hole in this columns row r, make the dots above it "falling"
			if(grid[c][r]==EMPTY)
			{
				// loop over the remaining dots in this row
				for(i=r; i>=0; i--)
				{
					// if it's not empty, spawn a falling dot from it
					if(grid[c][i]>=1 && grid[c][i]<=5)
					{
						// spawn a falling dot of the same colour, here
						Game_SpawnFallingDot(grid[c][i], c, i);

						// clear the dot from the grid
						// (falling dots aren't grid members until they land)
						grid[c][i] = EMPTY;

					}// end if dot found to make fall					

				}// next i
			
			}// end if empty hole found

		}// next r (row)

	}// next c (column)

}// void Game_CheckEmptyDotGaps()


/**
 * Spawns a falling dot
 *
 * @param color the color type of the dot to make falling
 * @param x the x position (column) of the dot to fall
 * @param y the current y position of the dot to fall. NOTE: if y==-1, the dot will spawn just above the first row
 */
void Game_SpawnFallingDot(int color, int x, int y)
{
	// due to limitation in GBDK and LCC, local vars always need to be defined on top
	int i;
	int fallingSlotIndex;

	// we need to find a suitable hole in our array of falling dots to spawn this dot
	fallingSlotIndex=0;
	for(i=0; i<30; i++)
	{
		// if the first value of the second dimension is 0, that slot is inactive and can be used
		if(fallingBois[i][FALLING_ACTIVE]==0)
		{
			// save slot to spawn falling dot
			fallingSlotIndex = i;

			// exit loop
			break;
		}
	}// next i

	// set up our new falling dot...

	// it's now active:
	fallingBois[fallingSlotIndex][FALLING_ACTIVE] = 1;

	// set it's color
	fallingBois[fallingSlotIndex][FALLING_COLOR] = color;

	// set it's column
	fallingBois[fallingSlotIndex][FALLING_COLUMN] = x;

	// convert it's row into a grid position and set it:
	i = (y*3) + 3;
	fallingBois[fallingSlotIndex][FALLING_ROW] = i;

	// if y is -1, that is a special number.
	// we should spawn the dot just one row above the top row (i.e. background grid row 2)
	if(y==-1)
		fallingBois[fallingSlotIndex][FALLING_ROW] = 2;

	// since we just spawned one, this is true now, regardless if it was true or not before
	gameHasFallingDots = 1;

}// void Game_SpawnFallingDot(int color, int x, int y)


/**
 * If there's falling dots, move em down, & stick the landing
 */
void Game_UpdateFallingDots()
{
	// due to limitation in GBDK and LCC, local vars always need to be defined on top
	int i;
	int foundFallingDot;
	int gridYSpot;

	// FOR DEBUG: show if we have falling dots or not, with a tile in the top left corner of the screen
	// if(gameHasFallingDots==1)
	// 	setBKGTile(0, 0, TL_LINE_HORIZ, 0);
	// else
	// 	setBKGTile(0, 0, TL_EMPTY, 0);

	// if the game currently doesn't have falling dots, just GTFO
	if(gameHasFallingDots==0)
		return;

	// don't update falling dots until we've waited a bit
	fallUpdateLimiter++;
	if(fallUpdateLimiter<10)
		return;

	// reset update wait timer
	fallUpdateLimiter = 0;

	// set this to false (0), unless we find at least one active one in the loop below
	foundFallingDot = 0;

	// loop over all our possible falling dots and update them if active
	for(i=0; i<30; i++)
	{
		// only upate this falling dot if active
		if(fallingBois[i][FALLING_ACTIVE]==1)
		{
			// we found a falling dot, so this is true from now on
			foundFallingDot = 1;

			// first, clear it's tiles on the background grid of tiles
			setBKGTile(fallingBois[i][FALLING_COLUMN]*3+2, fallingBois[i][FALLING_ROW], TL_EMPTY, 0);

			// move it's position down:
			fallingBois[i][FALLING_ROW]++;

			/*
				H'okay, so, if it's falling position lines up with a solid grid position...

				What we'll do is we'll temporarily kill this falling dot and add it back to the grid.
				Then, at the end of this function, well do another gap-check and spawn new falling dots.
				If there's still empty space under this dot, it will turn back into a falling dot anyway.
				If this dot is at the bottom or there's no empty space under it, it wont continue to fall

				Since the dots start at y position 3, and the grid is spaced in 3's, we can just use %3
				without additional math to check if it lines up in a correct vertical position.
			*/

			// check if our falling boi lines up with a vertical row on the grid
			if(fallingBois[i][FALLING_ROW]%3==0)
			{
				// kill 🔪🔪🔪 this falling dot:
				fallingBois[i][FALLING_ACTIVE] = 0;

				// determine which grid spot this this lines up into
				// NOTE: we subtract 1 after b/c the first position of 3 is 1 and that's 0 in the dot grid
				gridYSpot = fallingBois[i][FALLING_ROW] / 3;
				gridYSpot--;

				// set this falling dot's grid position just to be this dot, of it's color
				grid[fallingBois[i][FALLING_COLUMN]][gridYSpot] = fallingBois[i][FALLING_COLOR];

			}// endif falling dot lines up with grid row

			// regardless if this falling dot became part of the grid, or if it's still falling,
			// we should set it's background grid tile to this dot
			setBKGTile(fallingBois[i][FALLING_COLUMN]*3+2, fallingBois[i][FALLING_ROW], TL_DOT, fallingBois[i][FALLING_COLOR]);

		}// end if falling boi is active

	}// next i

	// if we didn't find ANY falling dot's in that entire pass, we can disable future falling updates
	// until another falling dot is spawned
	if(foundFallingDot==0)
		gameHasFallingDots = 0;

	// since we may have moved some falling dots back into the grid, they may still have empty
	// gap holes below them. Call this again - if there's no holes, no new falling dots will spawn
	Game_CheckEmptyDotGaps();

	// update valid cursor selection incase a dot fell in a valid place under the hand-pointer cursor
	Game_CheckValidPick();

	/*
		if we:
			- finished moving our dots
			- and we checked for empty dot gaps but didn't spawn any new falling dots
		BUT:
			- there are still gaps along the top, we should spawn some new random dots to fall

		Every time this method completes with no more falling dots we will call `Game_CheckEmptyTops()`
		to possibly filly empty dot-holes on the top row of the dot grid.

		Eventually this method will complete with no empty dot-holes on the top of the grid, and that
		means the entire grid is fleshy out with dots. No more work to do here, etc.
	*/
	if(gameHasFallingDots==0)
		Game_CheckEmptyTops();

}// void Game_UpdateFallingDots()


/**
 * When the falling dots have settled, check if there are empty holes on top row to spawn new falling dots.
 *
 * Note: this method should only be called from `Game_UpdateFallingDots()` when it finished
 * and it determined there are no more falling dots active, or from `Game_CheckEmptyDotGaps()` when
 * it checked all gaps, but no falling dots were spawned. The latter case means dots were cleared from
 * the dot grid's top-row and there were no dots "above" them to start falling.
 *
 * Since this method will spawn falling dots, Game_UpdateFallingDots() will continue to call on future loops.
 *
 * But, eventually all falling dots will stop and all top holes will be filled.
 */
void Game_CheckEmptyTops()
{
	// due to limitation in GBDK and LCC, local vars always need to be defined on top
	int x;
	UBYTE r;
	int randomDotColor;

	// loop over the top row on our grid.
	// if we find an empty hole, we can spawn a dot for that column
	for(x=0; x<6; x++)
	{
		// check if we found an empty top-spot
		if(grid[x][0]==EMPTY)
		{
			/*
				Below we pick a random new color for the dot we'll spawn.
				
				NOTE: for some reason rand() returns garbage unless I store it in a UBYTE first.
				This solution found from `rand.c` in GBDK example files.

				Also NOTE: right now there is no logic to make sure the spawned dots will guarantee
				valid future moves.

				In theory, it's currently possible to clear the grid into a state where no dot's have
				similarily coloured adjacent neighbors.

				It SHOULD be the job of newly spawned dots to pick a color, such that when they land
				in place on the dot grid, there is a valid move to select / chain.

				However, because I'm short on ROM space, and because this edge-case is very unlikely,
				I've knowling omitted this feature.

				From my play-testing, I've yet to arrive in such a situation that the dot-grid is
				unplayable before the 60 seconds runs out. It COULD happen, but it's so rare that
				I can't afford to sacrafice the ROM space to fix this edge case.

				As it stands, completely random coloured dot's will be spawned on top for now.

				Perhapse in the future if I have the ROM space available after all features are added,
				this bug will be patched out. But for now, this rare-edge case will go un fixed as a
				known-glitch but unlikely edge-case.
				
				Sorry to any player-bro's that make the dot grid unplayable before time is out.
			*/

			// pick random colour as UBYTE, and modulo that mofo to be between 0 and 5, exclusive.
			r = rand();
			randomDotColor = (int)(r%5);
			
			// our colors start at 1, since 0 is empty.
			// so we'll add 1 to the result of the mod 5 above
			randomDotColor++;

			// spawn a random new dot at this column, just above the top of the grid
			Game_SpawnFallingDot(randomDotColor, x, -1);

		}// end if found empty

	}// next x

}// void Game_CheckEmptyTops()


/**
 * Moves the cursor appropriately, based on the x/y spaces to move.
 *
 * @param x spaces to move cursor x position on grid
 * @param y spaces to move cursor y position on grid
 */
void Game_MoveCursor(int x, int y)
{
	/*
		Below we change cursor position.
		
		NOTE: `Game_MoveCursor(x, y)` is called exlusively from `Game_Input()` and the method
		`Game_Input()` will only ever pass a non-zero value into either x or y, but not both.

		Thus, it's not possible to move in diagonals.

		Below we can just add both x and y to the cursorGridX/cursorGridY because we always
		know that one of either x or y will always be 0 and the other will always be 1 or -1.
	*/
	cursorGridX += x;
	cursorGridY += y;

	// boundary check the hand-pointer cursor's position to make sure it's always withing the 6x5 grid
	if(cursorGridX<0)
		cursorGridX=0;
	if(cursorGridX>5)
		cursorGridX=5;
	if(cursorGridY<0)
		cursorGridY=0;
	if(cursorGridY>4)
		cursorGridY=4;
	
	/*
		Below we play sound based on game mode, to denote the movement.

		NOTE: we need the if() block's curlys ({}) here b/c the SND_STEP and SND_JUMP
		macros have multiple inline commands (; separated), so it needs a full block
		to play the sound.
	*/
	if(gameMode==MODE_PICK)
	{
		SND_STEP
	}
	else if(gameMode==MODE_MOVE)
	{
		SND_JUMP
	}

	/*
		This method, `Game_MoveCursor()` is called from both `Game_Input()` in MODE_PICK,
		and from `Game_MoveSelection(...)` in MODE_MOVE.

		While these checks only pertain to one-specific mode, either way they need to be
		called and it won't decrease rom-size to put these in mode-specific locations.

		As such, we'll update both our valid picks and moves always at the end of this
		method every time, all the time.
	*/
	Game_CheckValidPick();
	Game_CheckValidMoves();

}// void Game_MoveCursor(int x, int y)


/**
 * Makes sure the GameBoy's background tile grid is up-to-date with our logical grid's dots/empties
 *
 * @param forceUpdate set to 1 to skip the update timer
 */
void Game_UpdateGrid(int forceUpdate)
{
	// due to limitation in GBDK and LCC, local vars always need to be defined on top
	int x;
	BYTE y;
	int gridTile;
	BYTE bkgX;
	BYTE bkgY;
	int existingGridTile;

	// don't update grid until we've waited a bit
	gridUpdateLimiter++;
	if(gridUpdateLimiter<100 && forceUpdate!=1)
		return;

	// reset update wait
	gridUpdateLimiter = 0;

	/*
		Because our method `getBKGTile(bkgX, bkgY)` doesn't explicity set the VRAM bank to 0
		(the tile-data VRAM bank), to save on processing power, we'll make sure to explicitly
		set it to 0 here.

		Note that, we call `setBKGTile(...)` also in the loop below, but it always ends on
		switching to VRAM bank 0, so future calls to `getBKGTile(...)` will still be on 0.
	*/
	VBK_REG = 0;

	// loop over our grid and make sure GameBoy background tiles match grid data
	for(x=0; x<6; x++)
	{
		for(y=0; y<5; y++)
		{
			// calculate corresponding background tile position, using BYTES to avoid GBDK/LCC fuckery
			bkgX = x*3;
			bkgY = y*3;
			bkgX = (BYTE)bkgX + (BYTE)2;
			bkgY = (BYTE)bkgY + (BYTE)3;

			// get grid colour tile at this location, or 0 (EMPTY) if empty
			gridTile = grid[x][y];

			// get current tile at this dot grid location
			existingGridTile = getBKGTile(bkgX, bkgY);

			// if it's empty, set the tile to be empty
			if(gridTile==EMPTY)
				setBKGTile(bkgX, bkgY, TL_EMPTY, gridTile);
			else if(existingGridTile==TL_EMPTY)
				setBKGTile(bkgX, bkgY, TL_DOT, gridTile);

		}// next y

	}// next x

}// void Game_UpdateGrid(int forceUpdate)


/**
 * The main game loop. 'Nuff said.
 */
void Game_Loop()
{
	// loop until our mode is MODE_RESULTS which means the game time is up
	while(gameMode!=MODE_RESULTS)
	{
		// update our in game clock
		Game_UpdateTime();

		// get input from the GameBoy controller and act on it depending on `gameMode`
		Game_Input();

		// update sprite positions / statuses for our move-arrows or hand-pointer cursor.
		Game_UpdateSprites();

		// update the GameBoy's background grid tiles, to be either empty or solid dot's
		Game_UpdateGrid(0);

		// update falling dots animations on screen and set them in-place on the grid, if "landed"
		Game_UpdateFallingDots();

		/*
			If we have any big dots to eventually shrink, decrease their timer and/or shrink 'em.
			
			NOTE: maybe this should be moved to something like `Game_UpdateBigDots()` eventually,
			but due to ROM size concerns, this big-dot-update code will be left in-place for now.
		*/
		if(bigDotTimer!=0)
		{
			// decrease time for big dot to be visible
			bigDotTimer--;

			// if we're out of time, shrink our current big dot
			if(bigDotTimer==0)
				Game_ShrinkDot(bigDotGridX, bigDotGridY);
		
		}// end if big dot visible

	}// wend

	// show results and wait for start to be pressed
	Game_ResultsLoop();

}// void Game_Loop()


/**
 * When time has run out, the result loop runs instead.
 *
 * This method is the main loop for showing the end-of-game results screen, with your score,
 * best score, and flashing "PRESS A" text.
 *
 * Using the global variable `resultsSlide` this method will slide the GameBoy's tile background
 * up to "scroll the screen".
 *
 * It won't let the A button be valid press until the screen is scrolled all the way up.
 */
void Game_ResultsLoop()
{
	// due to limitation in GBDK and LCC, local vars always need to be defined on top
	int slideTimer;
	int pressATimer;
	int showPressA;

	// initialize the timer for the flashing "PRESS A" text and the boolean to show said text.
	pressATimer = 0;
	showPressA = 1;

	// make sure background is white no matter what
	Game_SetBGColor(EMPTY);

	// update best score:
	if(score>bestScore)
		bestScore = score;

	// update score and best score on GameBoy's background scoreboard tiles
	Game_DrawNumber(12, 22, score);
	Game_DrawNumber(11, 25, bestScore);

	/*
		Once time is up and we're in the MODE_RESULTS mode, the hand-pointer cursor,
		and move-arrow sprites shouldn't be visible any more, so below the macro
		will disable GameBoy's sprites layer.
	*/
	HIDE_SPRITES;

	// reset our loop counter timer for slide animation
	slideTimer = 0;

	// set VRAM bank to blink "press a" tiles
	VBK_REG = 0;

	// loop both to animate the screen slide, as well as wait until A is pressed
	while(!(joypad() & J_START) || resultsSlide<120)
	{
		// increment "PRESS A" flash timer, and toggle the "PRESS A" GameBoy background tiles
		pressATimer++;
		if(pressATimer>6000)
		{
			// reset "PRESS A" flash timer and toggle the visibility of the "PRESS A" text
			pressATimer=0;
			showPressA = (showPressA==1) ? 0 : 1;

			// depending if we should show "PRESS A" or not, update GameBoy background tiles
			if(showPressA==1)
				set_bkg_tiles(5, 29, 11, 1, press_start);		
			else
				set_bkg_tiles(5, 29, 11, 1, clear_line);

		}// end if press A toggle timer has elapsed (over 6K)

		// if we're still sliding
		if(resultsSlide<128)
		{
			// increase slide delay timer
			slideTimer++;

			// if we're over the wait time, reset the slide timer and scroll the BG 1px vertically
			if(slideTimer>30)
			{
				// reset scroll wait timer
				slideTimer=0;

				// increase slide value for bg, and slide the BG
				resultsSlide++;
				move_bkg(0, resultsSlide);
			
			}// end if time to slide
			
			// if our slide is at 100, we can clear HUD row on the BG so it doesn't loop up from the bottom
			if(resultsSlide==100)
				set_bkg_tiles( 0, 1, 20, 1, clear_line);
			
		}// end if still sliding

	}// wend

	// wait for A release
	while((joypad() & J_START));

}// void Game_ResulsLoop()