/*
	main.c
	------

	The main file / entry-point for the DOTs game.

	This game is built with the GBDK dev tool kit.

	Note about GBDK and it's compiler, LCC:

	The GameBoy and GameBoy Color hardware use a processor based on the Zilog Z80,
	that was customized by Nintendo to include game-specific OP codes.

	This processor was NEVER meant to run code compiled from C. All original GameBoy
	Color games were written fully in Assembly, as they should.

	As such, the compiler -- LCC -- is slightly buggy. Sometimes, perfectly valid
	code will compile but NOT run on the GameBoy.

	Other times, the compiler will generate warnings or errors despite the code
	being logically flawless.

	This is mostly due to the fact that the GameBoy's Z80 is an 8-Bit CPU, and it has
	very few registers / stack space.

	Every Integer operation that is over 8-Bit will expand to quite a large amount
	of Assembly Instructions (i.e. OP Codes). This makes simple things very expensive.
	Multiplication, and especially division, are noticably expensive.

	But another side effect of targeting an 8-Bit CPU with limited registers is that
	complex nested code or inline code sometimes fails to compile correctly.

	Often without warning.

	So, something like:

		int foo = (x*5) + (y*10) + 8;

	Might actually be TOO many operations in one line!

	Sometimes, rewriting the above code like this:

		int ax = x * 5;
		int ay = y * 10;
		int foo = ax + ay;
		foo += 8;

	Will compile correctly, even though it's logically the same as above.

	But because we deliberately make variables for some of the sub-operations,
	this acts as a replacement to the assembly registers/stacks, via variables.

	So, the code for this project is... written weirdly, to say the least.

	During this project I was often fighting the compiler, and refactoring code
	to "flatten" it or break it out into multiple steps that wouldn't be necessary
	on any other platform.

	Also, nested operations like multiple nested for-loops or if() statements
	might also be too much for the compiler to generate assembly. In these cases,
	I had to flatten out loops or write them in non-intuitive ways to avoide too
	much nesting.

	Due to the fact that we're dealing with an 8-Bit CPU and integer operations
	(i.e. operations on int types larger than a BYTE) expand out to a lot of assembly,
	sometimes simple integer operations are quirky.

	For instance: logically `i>0` and `i>=1` are the same.

	But sometimes GBDK will fail to compare integers correctly and `i>0` will need
	to be swapped out for `i>=1` in loops or if()'s, or visa-versa.

	This can be especially tricky to debug but I feel I've handled these cases -
	but you might notice through-out the codebase my use of non-standard comparison
	operators to get around this quirk.

	Another quirk of LCC is that it cannot define variables inline.

	Variables must be defined as globals at the top of a file, or, at the top
	of the function they're defined in.

	So, for instance:

		for(int i=0; i<10; i++)
			...

	Would actually generate a compile error! Can you spot why?

	It's because the `int i` is INSIDE a foor loop! You can't define an new int, i,
	anywhere besides:
		- the top of the file
		- or the top of a method

	So, instead we must write:

		void someFunction()
		{
			// declare all locals at top of function:
			int i;

			// now do a loop:
			for(i=0; i<10; i++)
				...
		}

	In most cases I wrote a comment to note this quirk at the top of functions,
	as a reminder as to why all the variables are being defined upfront.
	
	One final note:

	C doesn't have any sort-of Class implementation.

	While I COULD use structs in some cases to tidy-code, structs can sometimes be
	expensive on rom/ram-size when passing ByVal.

	Thus, my code relies on a number of global variables, but in some cases my
	naming convention prefixes these globals with the file / code area they belong
	to, but not always. Many globals in game.c are just named for their purpose.

	However, most functions do adhere to this naming convention.

	Thus, a function starting with 'Game_' is defined in game.c and operates
	exclusively to the game code defined in game.c.

	Meanwhile, functions like `TitleScreen_Start()` are defined and used in
	titlescreen.c, where the code for the titlescreen logic lives.

	And the function `Main_ClearBG()` lives in main.c as a general function to clear
	the GameBoy's background tiles - to be used everywhere in code.

	So, despite the lack of Classes and extensive use of globals, my code is
	organized with prefixes such as "Main_", "Game_", and "TitleScreen_", to denote
	where the function is defined and used.
	`
	--------------------------------------------------------------------------------

	To summarize:

	GBDK and it's compiller, LCC, are quikry due to the limitations of an 8-Bit
	Z80-based processor and it's limited registers.

	In many cases my code is written... weirdly. I apologize.

	If you're reading this, take note of the above examples. After working with
	GBDK and LCC for awhile you can develope a groove and "sixth sense" for when
	code might behave oddly and keeping these tactics in mind can keep you sane
	and keep the code compiling.

	As such, this is NOT by best code and NOT an example of good engineering.

	This code base is deliberately written to work around the limitaions of compiling
	C for GameBoy Color.

	This code can be whacky at times. You've been warned!
*/

// function declarations
void Init_loadPalettesAndTiles();
void Main_ClearBG();


// main GBDK and C includes
#include <gb/gb.h>
#include <stdio.h>
#include <gb/drawing.h>
#include <rand.h>

// include our tile sets
#include "tiles/main_tiles.h"
#include "tiles/main_tiles.c"

// include our various tile maps
#include "maps/clear_line.c"
#include "maps/clear_line_attr.c"
#include "maps/dots_logo.c"
#include "maps/dots_logo_attr.c"
#include "maps/press_start.c"
#include "maps/press_start_attr.c"
#include "maps/time_and_score.c"
#include "maps/time_and_score_attr.c"
#include "maps/results.c"
#include "maps/results_attr.c"
#include "maps/score.c"
#include "maps/score_attr.c"
#include "maps/best.c"
#include "maps/best_attr.c"
#include "maps/paused.c"
#include "maps/paused_attr.c"

// include our master list of defines for our sprites, tiles, and etc.
#include "defines.h"

// include code for title screen logic
#include "titlescreen.c"

// include main game logic code
#include "game.c"


/**
 * Clears the background screen by looping over the 18 vertical rows and copying our 20-wide clear line data
 */
void Main_ClearBG()
{
	// due to limitation in GBDK and LCC, local vars always need to be defined on top
	int i;

	// loop twice to copy our blank line to clear the bg screen area on both VRAM buffers
	VBK_REG = 1;
	for(i=0; i<18; i++)
		set_bkg_tiles(0, i, 20, 1, clear_line_attr);
	VBK_REG = 0;
	for(i=0; i<18; i++)
		set_bkg_tiles(0, i, 20, 1, clear_line);
	
}// void Main_ClearBG()


/**
 * Loads the pallets and tile sets for the background and sprites
 */
void Init_loadPalettesAndTiles()
{
	// load bg and sprite palettes
	set_bkg_palette(0, 8, bgTilePalettes);
	set_sprite_palette(0, 8, bgTilePalettes);
 
	// load bg and sprite tiles
	set_bkg_data(0x0, 60, MainTiles);
	set_sprite_data(0x0, 60, MainTiles);

}// void Init_loadPalettesAndTiles()


/** 
 * you know whattitiisss
*/
void main()
{   
	// Initialize sprite palette
	OBP1_REG = 0xE0U;

	// start sound!
	NR52_REG = 0x80;
	NR51_REG = 0xFB;
	NR50_REG = 0x77;

	// initialize our background tiles, sprite tiles and their palettes
	Init_loadPalettesAndTiles();

	// our main loop should toggle between the title screen and the game play!
	while(1)
	{
		// start with display off, title screen will turn it on when ready
		DISPLAY_OFF;

		// start title screen section
		TitleScreen_Start();

		// turn off again until the game turns it back on
		DISPLAY_OFF;

		// when the title screen loop exists, we can start the game, so lets initialize it's tiles
		Init_loadPalettesAndTiles();

		// start the game mode!
		Game_Start();

		// when the game mode loop exits, we can loop around to the title screen again
	
	}// wend

}// void main()
