/*
	titlescreen.c
	-------------

	This file handles the main loop and logic for the title screen.

	Here we set up the graphics for the title screen (i.e. the DOTS logo and 
	"PRESS START" text).

	We also periodically blink the "PRESS START" graphics and wait for start
	to be pressed before exiting, allow the main loop in main.c to continue
	to the game code.
*/


// function declarations to avoid implicit definition compiler warnings
void TitleScreen_Start();
void TitleScreen_Loop();


// the main blink speed / time length
#define BLINK_LENGTH 40

// show PRESS START or not?
BYTE TitleScreen_showPressStart;

// main timer for blinking the "PRESS START"
BYTE TitleScreen_blinkTimer;


/**
 * Sets up and starts the title screen sequence
*/
void TitleScreen_Start()
{
	// set / init title screen blink variables
	TitleScreen_showPressStart = TRUE;
	TitleScreen_blinkTimer = 0;

	// clear our background tiles
	Main_ClearBG();

	// copy dots logo attributes and tile data to both VRAM buffers
	VBK_REG = 1;
	set_bkg_tiles(5, 5, 11, 3, dots_logo_attr);
	VBK_REG = 0;
	set_bkg_tiles(5, 5, 11, 3, dots_logo);

	// title screen always top left, but scrolled slightly left to center the logo perfectly
	move_bkg(3, 0);

	// ready to show the title screen!
	DISPLAY_ON;

	// tells GBC we don't want sprites or the window visible, but we do want the BG visible
	HIDE_SPRITES;
	HIDE_WIN;
	SHOW_BKG;

	// enter main title screen logic loop
	TitleScreen_Loop();

}// void TitleScreen_Start()


/**
 * Main title screen logic loop
*/
void TitleScreen_Loop()
{
	// set register for animating the blinking "press start"
	VBK_REG = 0;

	// loop until start is pressed
	while(!(joypad() & J_START))
	{
		// increase blink timer, and hide / show "PRESS START" based on timer
		TitleScreen_blinkTimer++;
		if(TitleScreen_blinkTimer>BLINK_LENGTH)
		{
			TitleScreen_blinkTimer=0;
			TitleScreen_showPressStart = (TitleScreen_showPressStart==TRUE) ? FALSE : TRUE;
		}

		// if the "PRESS START" text is toggled on via TitleScreen_showPressStart, show it
		if(TitleScreen_showPressStart==TRUE)
			set_bkg_tiles(5, 12, 11, 1, press_start);
		else
			set_bkg_tiles(5, 12, 11, 1, clear_line);
		
		// wait for the next vertical blank
		// this lets the gameboy draw it's video memory when it get's a chance
		wait_vbl_done();

	}// wend

	// since the previous loop waited for start to be pressed,
	// we will enter another loop until start is released before
	// we continue to the game
	while((joypad() & J_START));

	// play start sound effect
	SND_START

}// void TitleScreen_Loop()
